/**
 * Copyright (c) 2011 Daniel Berenguer <dberenguer@usapiens.com>
 * 
 * This file is part of the panStamp project.
 * 
 * panStamp  is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * any later version.
 * 
 * panLoader is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with panLoader; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 
 * USA
 * 
 * Author: Daniel Berenguer
 * Creation date: 03/03/2011
 */

#include "cc1101.h"
#include "nvolat.h"
#include "serialcom.h"
/**
 * Macros
 */
// Select (SPI) CC1101
#define cc1101_Select()  bitClear(PORT_SPI_SS, BIT_SPI_SS)
// Deselect (SPI) CC1101
#define cc1101_Deselect()  bitSet(PORT_SPI_SS, BIT_SPI_SS)
// Wait until SPI MISO line goes low
#define wait_Miso()  while(bitRead(PORT_SPI_MISO, BIT_SPI_MISO))
// Get GDO0 pin state
#define getGDO0state()  bitRead(PORT_GDO0, BIT_GDO0)
// Wait until GDO0 line goes high
#define wait_GDO0_high()  while(!getGDO0state())
// Wait until GDO0 line goes low
#define wait_GDO0_low()  while(getGDO0state())
// Read CC1101 Config register
#define readConfigReg(regAddr)    readRegCC1101(regAddr, CC1101_CONFIG_REGISTER)
// Read CC1101 Status register
#define readStatusReg(regAddr)    readRegCC1101(regAddr, CC1101_STATUS_REGISTER)

 /**
  * PATABLE
  */
const byte paTable[8] = {0x60, 0x60, 0x60, 0x60, 0x60, 0x60, 0x60, 0x60};

/**
 * wakeUp
 * 
 * Wake up CC1101 from Power Down state
 */
void wakeUpCC1101(void)
{
  cc1101_Select();                      // Select CC1101
  wait_Miso();                          // Wait until MISO goes low
  cc1101_Deselect();                    // Deselect CC1101
}

/**
 * writeReg
 * 
 * Write single register into the CC1101 IC via SPI
 * 
 * 'regAddr'	Register address
 * 'value'	Value to be writen
 */
void writeRegCC1101(byte regAddr, byte value) 
{
  cc1101_Select();                      // Select CC1101
  wait_Miso();                          // Wait until MISO goes low
  sendspi(regAddr);                    // Send register address
  sendspi(value);                      // Send value
  cc1101_Deselect();                    // Deselect CC1101
}

/**
 * writeBurstReg
 * 
 * Write multiple registers into the CC1101 IC via SPI
 * 
 * 'regAddr'	Register address
 * 'buffer'	Data to be writen
 * 'len'	Data length
 */
void writeBurstRegCC1101(byte regAddr, byte* buffer, byte len)
{
  byte addr, i;
  
  addr = regAddr | WRITE_BURST;         // Enable burst transfer
  cc1101_Select();                      // Select CC1101
  wait_Miso();                          // Wait until MISO goes low
  sendspi(addr);                       // Send register address
  
  for(i=0 ; i<len ; i++)
    sendspi(buffer[i]);                // Send value

  cc1101_Deselect();                    // Deselect CC1101  
}

/**
 * cmdStrobe
 * 
 * Send command strobe to the CC1101 IC via SPI
 * 
 * 'cmd'	Command strobe
 */     
void cmdStrobeCC1101(byte cmd) 
{
  cc1101_Select();                      // Select CC1101
  wait_Miso();                          // Wait until MISO goes low
  sendspi(cmd);                        // Send strobe command
  cc1101_Deselect();                    // Deselect CC1101 
}

/**
 * readReg
 * 
 * Read CC1101 register via SPI
 * 
 * 'regAddr'	Register address
 * 'regType'	Type of register: CC1101_CONFIG_REGISTER or CC1101_STATUS_REGISTER
 * 
 * Return:
 * 	Data byte returned by the CC1101 IC
 */
byte readRegCC1101(byte regAddr, byte regType) 
{
  byte addr, val;

  addr = regAddr | regType;
  cc1101_Select();                      // Select CC1101
  wait_Miso();                          // Wait until MISO goes low
  sendspi(addr);                       // Send register address
  val = sendspi(0x00);                 // Read result
  cc1101_Deselect();                    // Deselect CC1101

  return val;
}

/**
 * readBurstReg
 * 
 * Read burst data from CC1101 via SPI
 * 
 * 'buffer'	Buffer where to copy the result to
 * 'regAddr'	Register address
 * 'len'	Data length
 */
void readBurstRegCC1101(byte * buffer, byte regAddr, byte len) 
{
  byte addr, i;
  
  addr = regAddr | READ_BURST;
  cc1101_Select();                      // Select CC1101
  wait_Miso();                          // Wait until MISO goes low
  sendspi(addr);                       // Send register address
  for(i=0 ; i<len ; i++)
    buffer[i] = sendspi(0x00);         // Read result byte by byte
  cc1101_Deselect();                    // Deselect CC1101
}

/**
 * reset
 * 
 * Reset CC1101
 */
void resetCC1101(void) 
{
  cc1101_Deselect();                    // Deselect CC1101
  _delay_us(5);
  cc1101_Select();                      // Select CC1101
  _delay_us(10);
  cc1101_Deselect();                    // Deselect CC1101
  _delay_us(41);
  cc1101_Select();                      // Select CC1101

  wait_Miso();                          // Wait until MISO goes low
  sendspi(CC1101_SRES);                // Send reset command strobe
  wait_Miso();                          // Wait until MISO goes low

  cc1101_Deselect();                    // Deselect CC1101

  setDefaultRegsCC1101();                     // Reconfigure CC1101
  setRegsFromEepromCC1101();                  // Take user settings from EEPROM
}

/**
 * setDefaultRegs
 * 
 * Configure CC1101 registers
 */
void setDefaultRegsCC1101(void) 
{
  byte defSyncWrd[] = {CC1101_DEFVAL_SYNC1, CC1101_DEFVAL_SYNC0};

  writeRegCC1101(CC1101_IOCFG2,  CC1101_DEFVAL_IOCFG2);
  writeRegCC1101(CC1101_IOCFG1,  CC1101_DEFVAL_IOCFG1);
  writeRegCC1101(CC1101_IOCFG0,  CC1101_DEFVAL_IOCFG0);
  writeRegCC1101(CC1101_FIFOTHR,  CC1101_DEFVAL_FIFOTHR);
  writeRegCC1101(CC1101_PKTLEN,  CC1101_DEFVAL_PKTLEN);
  writeRegCC1101(CC1101_PKTCTRL1,  CC1101_DEFVAL_PKTCTRL1);
  writeRegCC1101(CC1101_PKTCTRL0,  CC1101_DEFVAL_PKTCTRL0);

  // Set default synchronization word
  setSyncWordCC1101(defSyncWrd, false);

  // Set default device address
  setDevAddressCC1101(CC1101_DEFVAL_ADDR, false);
  // Set default frequency channel
  setChannelCC1101(CC1101_DEFVAL_CHANNR, false);
  
  writeRegCC1101(CC1101_FSCTRL1,  CC1101_DEFVAL_FSCTRL1);
  writeRegCC1101(CC1101_FSCTRL0,  CC1101_DEFVAL_FSCTRL0);

  // Set default carrier frequency = 868 MHz
  setCarrierFreqCC1101(CFREQ_868);

  writeRegCC1101(CC1101_MDMCFG4,  CC1101_DEFVAL_MDMCFG4);
  writeRegCC1101(CC1101_MDMCFG3,  CC1101_DEFVAL_MDMCFG3);
  writeRegCC1101(CC1101_MDMCFG2,  CC1101_DEFVAL_MDMCFG2);
  writeRegCC1101(CC1101_MDMCFG1,  CC1101_DEFVAL_MDMCFG1);
  writeRegCC1101(CC1101_MDMCFG0,  CC1101_DEFVAL_MDMCFG0);
  writeRegCC1101(CC1101_DEVIATN,  CC1101_DEFVAL_DEVIATN);
  writeRegCC1101(CC1101_MCSM2,  CC1101_DEFVAL_MCSM2);
  writeRegCC1101(CC1101_MCSM1,  CC1101_DEFVAL_MCSM1);
  writeRegCC1101(CC1101_MCSM0,  CC1101_DEFVAL_MCSM0);
  writeRegCC1101(CC1101_FOCCFG,  CC1101_DEFVAL_FOCCFG);
  writeRegCC1101(CC1101_BSCFG,  CC1101_DEFVAL_BSCFG);
  writeRegCC1101(CC1101_AGCCTRL2,  CC1101_DEFVAL_AGCCTRL2);
  writeRegCC1101(CC1101_AGCCTRL1,  CC1101_DEFVAL_AGCCTRL1);
  writeRegCC1101(CC1101_AGCCTRL0,  CC1101_DEFVAL_AGCCTRL0);
  writeRegCC1101(CC1101_WOREVT1,  CC1101_DEFVAL_WOREVT1);
  writeRegCC1101(CC1101_WOREVT0,  CC1101_DEFVAL_WOREVT0);
  writeRegCC1101(CC1101_WORCTRL,  CC1101_DEFVAL_WORCTRL);
  writeRegCC1101(CC1101_FREND1,  CC1101_DEFVAL_FREND1);
  writeRegCC1101(CC1101_FREND0,  CC1101_DEFVAL_FREND0);
  writeRegCC1101(CC1101_FSCAL3,  CC1101_DEFVAL_FSCAL3);
  writeRegCC1101(CC1101_FSCAL2,  CC1101_DEFVAL_FSCAL2);
  writeRegCC1101(CC1101_FSCAL1,  CC1101_DEFVAL_FSCAL1);
  writeRegCC1101(CC1101_FSCAL0,  CC1101_DEFVAL_FSCAL0);
  writeRegCC1101(CC1101_RCCTRL1,  CC1101_DEFVAL_RCCTRL1);
  writeRegCC1101(CC1101_RCCTRL0,  CC1101_DEFVAL_RCCTRL0);
  writeRegCC1101(CC1101_FSTEST,  CC1101_DEFVAL_FSTEST);
  writeRegCC1101(CC1101_PTEST,  CC1101_DEFVAL_PTEST);
  writeRegCC1101(CC1101_AGCTEST,  CC1101_DEFVAL_AGCTEST);
  writeRegCC1101(CC1101_TEST2,  CC1101_DEFVAL_TEST2);
  writeRegCC1101(CC1101_TEST1,  CC1101_DEFVAL_TEST1);
  writeRegCC1101(CC1101_TEST0,  CC1101_DEFVAL_TEST0);
}

/**
 * init
 * 
 * Initialize CC1101
 */
void initCC1101(void) 
{
  initspi();                           // Initialize SPI interface
  DDRD &=~(_BV(GDO0));//pinMode(GDO0, INPUT); // Config GDO0 as input //<-----------------------  A changer

  resetCC1101();                              // Reset CC1101

  // Configure PATABLE
  writeBurstRegCC1101(CC1101_PATABLE, (byte*)paTable, 8);
}

/**
 * setSyncWord
 * 
 * Set synchronization word
 * 
 * 'sync'	Synchronization word
 * 'save' If TRUE, save parameter in EEPROM
 */
void setSyncWordCC1101(byte *sync, bool save) 
{
  if ((cc1101.syncWord[0] != sync[0]) || (cc1101.syncWord[1] != sync[1]))
  {
    writeRegCC1101(CC1101_SYNC1, sync[0]);
    writeRegCC1101(CC1101_SYNC0, sync[1]);
    memcpy(cc1101.syncWord, sync, sizeof(cc1101.syncWord));
    // Save in EEPROM
    if (save)
    {
      writeEEPROM(EEPROM_SYNC_WORD, sync[0]);
      writeEEPROM(EEPROM_SYNC_WORD + 1, sync[1]);
    }
  }
}

void setSyncWordCC1101_n3(uint8_t syncH, uint8_t syncL, bool save) 
{
  save=false;

  if ((cc1101.syncWord[0] != syncH) || (cc1101.syncWord[1] != syncL))
  {
    writeRegCC1101(CC1101_SYNC1, syncH);
    writeRegCC1101(CC1101_SYNC0, syncL);
    cc1101.syncWord[0] = syncH;
    cc1101.syncWord[1] = syncL;
    // Save in EEPROM
    if (save)
    {
      writeEEPROM(EEPROM_SYNC_WORD, syncH);
      writeEEPROM(EEPROM_SYNC_WORD + 1, syncL);
    }
  }

   writeRegCC1101(CC1101_SYNC1, syncH);
   writeRegCC1101(CC1101_SYNC0, syncL); 

   cc1101.syncWord[0] = syncH;
   cc1101.syncWord[1] = syncL;
}




/**
 * setDevAddress
 * 
 * Set device address
 * 
 * 'addr'	Device address
 * 'save' If TRUE, save parameter in EEPROM
 */
void setDevAddressCC1101(byte addr, bool save) 
{
  if (cc1101.devAddress != addr)
  {
    writeRegCC1101(CC1101_ADDR, addr);
    cc1101.devAddress = addr;
    // Save in EEPROM
    if (save)
      writeEEPROM(EEPROM_DEVICE_ADDR, addr);  
  }
}

/**
 * setChannel
 * 
 * Set frequency channel
 * 
 * 'chnl'	Frequency channel
 * 'save' If TRUE, save parameter in EEPROM
 */
void setChannelCC1101(byte chnl, bool save) 
{
  if (cc1101.channel != chnl)
  {
    writeRegCC1101(CC1101_CHANNR,  chnl);
    cc1101.channel = chnl;
    // Save in EEPROM
    if (save)
      writeEEPROM(EEPROM_FREQ_CHANNEL, chnl);
  }
}

/**
 * setCarrierFreq
 * 
 * Set carrier frequency
 * 
 * 'freq'	New carrier frequency
 */
void setCarrierFreqCC1101(byte freq)
{
  switch(freq)
  {
    case CFREQ_915:
      writeRegCC1101(CC1101_FREQ2,  CC1101_DEFVAL_FREQ2_915);
      writeRegCC1101(CC1101_FREQ1,  CC1101_DEFVAL_FREQ1_915);
      writeRegCC1101(CC1101_FREQ0,  CC1101_DEFVAL_FREQ0_915);
      break;
    default:
      writeRegCC1101(CC1101_FREQ2,  CC1101_DEFVAL_FREQ2_868);
      writeRegCC1101(CC1101_FREQ1,  CC1101_DEFVAL_FREQ1_868);
      writeRegCC1101(CC1101_FREQ0,  CC1101_DEFVAL_FREQ0_868);
      break;
  }
   
  cc1101.carrierFreq = freq;  
}

/**
 * setRegsFromEeprom
 * 
 * Set registers from EEPROM
 */
void setRegsFromEepromCC1101(void)
{
  byte bVal;
  byte arrV[2];

  // Read RF channel from EEPROM
  bVal = readEEPROM(EEPROM_FREQ_CHANNEL);
  // Set RF channel
  if (bVal < NUMBER_OF_FCHANNELS )
    setChannelCC1101(bVal, false);
  // Read Sync word from EEPROM
  arrV[0] = readEEPROM(EEPROM_SYNC_WORD);
  arrV[1] = readEEPROM(EEPROM_SYNC_WORD + 1);
  // Set Sync word. 0x00 and 0xFF values are not allowed
  if (((arrV[0] != 0x00) && (arrV[0] != 0xFF)) || ((arrV[1] != 0x00) && (arrV[1] != 0xFF)))
    setSyncWordCC1101(arrV, false);
  // Read device address from EEPROM
  bVal = readEEPROM(EEPROM_DEVICE_ADDR);
  // Set device address
  if (bVal > 0)
    setDevAddressCC1101(bVal, false);
}

/**
 * setPowerDownState
 * 
 * Put CC1101 into power-down state
 */
void setPowerDownStateCC1101() 
{
  // Comming from RX state, we need to enter the IDLE state first
  cmdStrobeCC1101(CC1101_SIDLE);
  // Enter Power-down state
  cmdStrobeCC1101(CC1101_SPWD);
}

/**
 * sendData
 * 
 * Send data packet via RF
 * 
 * 'packet'	Packet to be transmitted
 *
 *  Return:
 *    True if the transmission succeeds
 *    False otherwise
 */
boolean sendDataCC1101(CCPACKET packet)
{
  byte marcState;

  // Enter RX state
  setRxState();

  // Check that the RX state has been entered
  while ((readStatusReg(CC1101_MARCSTATE) & 0x1F) != 0x0D)
  _delay_ms(1000); 
  
  

  _delay_us(500);//delayMicroseconds(500);

  // Set data length at the first position of the TX FIFO
  writeRegCC1101(CC1101_TXFIFO,  packet.length);
  // Write data into the TX FIFO
  writeBurstRegCC1101(CC1101_TXFIFO, packet.data, packet.length);

  // CCA enabled: will enter TX state only if the channel is clear
  //cmdStrobe(CC1101_STX);
  setTxState();

  // Check that TX state is being entered (state = RXTX_SETTLING)
  marcState = readStatusReg(CC1101_MARCSTATE) & 0x1F;
  if((marcState != 0x13) && (marcState != 0x14) && (marcState != 0x15))
  {
    setIdleState();       // Enter IDLE state
    flushTxFifo();        // Flush Tx FIFO
    setRxState();         // Back to RX state
    return false;
  }

  // Wait for the sync word to be transmitted
  wait_GDO0_high();

  // Wait until the end of the packet transmission
  wait_GDO0_low();

  // Enter back into RX state
  setRxState();

  // Check that the TX FIFO is empty
  if((readStatusReg(CC1101_TXBYTES) & 0x7F) == 0)
    return true;

  return false;
}

/**
 * receiveData
 * 
 * Read data packet from RX FIFO
 * 
 * Return:
 * 	Amount fo bytes received
 * 'packet'	Container for the packet received
 */
byte receiveDataCC1101(CCPACKET * packet)
{
  byte val;

  // Rx FIFO overflow?
  if ((readStatusReg(CC1101_MARCSTATE) & 0x1F) == 0x11)
  {
    setIdleState();       // Enter IDLE state
    flushRxFifo();        // Flush Rx FIFO
    packet->length = 0;
  }
  // Any byte waiting to be read?
  else if (readStatusReg(CC1101_RXBYTES) & 0x7F)
  {
    // Read data length
    packet->length = readConfigReg(CC1101_RXFIFO);
    // If packet is too long
    if (packet->length > CC1101_DATA_LEN)
      packet->length = 0;   // Discard packet
    else
    {
      // Read data packet
      readBurstRegCC1101(packet->data, CC1101_RXFIFO, packet->length);
      // Read RSSI
      packet->rssi = readConfigReg(CC1101_RXFIFO);
      // Read LQI and CRC_OK
      val = readConfigReg(CC1101_RXFIFO);
      packet->lqi = val & 0x7F;
      packet->crc_ok = bitRead(val, 7);
    }
  }
  else
    packet->length = 0;

  // Back to RX state
  setRxState();

  return packet->length;
}


//##################### Lecture de la commande recu #################################
int lectcommande(CCPACKET * packet)
{

if( (packet->data[1]=='C') && (packet->data[2]=='A') && (packet->data[3]=='L') && (packet->data[4]=='I') && (packet->data[5]=='B'))
      {return 1;} 


if( (packet->data[1]=='L') && (packet->data[2]=='E') && (packet->data[3]=='C')&& (packet->data[4]=='T'))  
      {return 2;}

if ((packet->data[1]=='A') && (packet->data[2]=='C') && (packet->data[3]=='C')&& (packet->data[4]=='U') && (packet->data[4]=='M'))     
      {return 3;}

if ((packet->data[1]=='O') && (packet->data[2]=='K'))
      {return 4;}

if( (packet->data[1]=='C') && (packet->data[2]=='A') && (packet->data[3]=='L') && (packet->data[4]=='I') && (packet->data[5]=='B')&& (packet->data[6]=='O')&& (packet->data[7]=='K'))
      {return 5;}

if( (packet->data[1]=='D') && (packet->data[2]=='A') && (packet->data[3]=='T')&& (packet->data[4]=='A'))  
      {return 6;}

return 8;
}

//######################### Reponse de prise en compte de l'ésclave (OK,CALIBOK) #######################
void AnswerSlave()
{ 
  CCPACKET packetok;

packetok.length=4;
packetok.data[0]=5; // adresse de destination
packetok.data[1]=0x4F; //O
packetok.data[2]=0x4B; //K
packetok.data[3]='0';

  sendDataCC1101(packetok);
      _delay_ms(200);
}

void trameconfirmationcalibration()
{
  CCPACKET packetcalibok;
  int i;
  int j=0;
  char calibration[8]="CALIBOK";

packetcalibok.length=9;
packetcalibok.data[0]=5; // adresse de destination
packetcalibok.data[3]='0';

for(i=1;i<packetcalibok.length;i++) //constitution du paquet
  {
  packetcalibok.data[i]=calibration[j];
  j+=1;
  }

  sendDataCC1101(packetcalibok);
   _delay_ms(200);
}


void envoiDATA(uint8_t *data)
{
  CCPACKET packetdata;
  int i;
  int j=0;
  char commande[5]="DATA";

packetdata.length=15;
packetdata.data[0]=5; // adresse de destination
packetdata.data[15]='0';

//constitution du paquet
for(i=1;j<=3;i++) 
  {
  packetdata.data[i]=commande[j];
  j+=1;
  }

  j=0;

for(i=5;i<packetdata.length;i++) 
  {
  packetdata.data[i]=data[j];
  j+=1;
  }

  sendDataCC1101(packetdata);
   _delay_ms(200);
}


//###################### fonctions pour constituer la trame de demande MASTER (calibration,lecture)###########
void tramecalibconstitution(CCPACKET * packet)
{
  int i;
  int j=0;
  char calibration[6]="CALIB";

for(i=1;i<packet->length;i++) //constitution du paquet
  {
  packet->data[i]=calibration[j];
  j+=1;
  }
}

void tramelecture(uint8_t addr)
{ CCPACKET packetlect;
  int i;
  int j=0;
  char commandelect[5]="LECT";

packetlect.length=6;
packetlect.data[0]=addr; // adresse de destination
packetlect.data[5]='0';

for(i=1;i<packetlect.length;i++) //constitution du paquet
  {
  packetlect.data[i]=commandelect[j];
  j+=1;
  }
  sendDataCC1101(packetlect);
   _delay_ms(200);
}



//################ fonction permettant de spécifier le courant , la tension et la température de calibration au recepteur ###########
void datacalibration(CCPACKET * packet)
{
   char *courant,*tension,*temperature;
   char tampon[4]={0};
   char tampon2[4]={0};
   char tampon3[4]={0};
    
   uint8_t val1, val2, val3;

   char datarequest[]="tapez le courant I , la tension V et la temperature T a calibrer. Tapez entrer entre chaques variables";

   //initialisation des pointeurs
   courant=&tampon[0];
   tension=&tampon2[0];
   temperature=&tampon3[0];

   getByte();
   writeString(datarequest);

   //courant
   val1=0; val2=0; val3=0;
  while((*courant = getByte()) != '\n')
  {++courant;} 
   //chiffre en code ascii-48 correspond à la vrai valeur
   val1=tampon[0]-48;  val2=tampon[1]-48;  //val3=tampon[2]-48;
   //association des valeurs pour obtenir le chiffre 
   uint8_t intensite = 10*val1+val2;//val3;   <---------------------------modification
   //putByte(intensite);
    
   //tension
    val1=0; val2=0; val3=0;
   while((*tension = getByte()) != '\n')
   {++tension;}
   //chiffre en code ascii-48 correspond à la vrai valeur
   val1=tampon2[0]-48;  val2=tampon2[1]-48;  val3=tampon2[2]-48;
   //association des valeurs pour obtenir le chiffre 
   uint8_t voltage = 100*val1+10*val2+val3;

   //temperature
   val1=0; val2=0; val3=0;
   while((*temperature = getByte()) != '\n')
   {++temperature;}
   //chiffre en code ascii-48 correspond à la vrai valeur
   val1=tampon3[0]-48;  val2=tampon3[1]-48;  //val3=tampon3[2]-48;
   //association des valeurs pour obtenir le chiffre 
   uint8_t tempera = 10*val1+val2;//+val3;

   

CCPACKET packetcalib;
  int i;
  int j=0;
  char commandelect[6]="CALIB";

packetcalib.length=10;
packetcalib.data[0]=4; // adresse de destination
packetcalib.data[9]='0';

for(i=1;j<=4;i++) //constitution du paquet
  {
  packetcalib.data[i]=commandelect[j];
  j+=1;
  }
   
   packetcalib.data[6]=intensite;
   packetcalib.data[7]=voltage;
   packetcalib.data[8]=tempera;


  sendDataCC1101(packetcalib);
   _delay_ms(200);
}


//permet d'afficher les valeurs VRMS, IRMS ,PRMS sur le port serie du MASTER
void affichagevaleurs(CCPACKET *packet)
{
//arrondir les valeurs de 3octets en 1 octet
char tensioninformation[]="La tension est:\n";
char courantinformation[]="Le courant est:\n";
char puissanceinformation[]="La puissance est:\n";

writeString(tensioninformation);
//send_serial(tension);
writeString(courantinformation); 
//send_serial(courant);
writeString(puissanceinformation);
//send_serial(puissance);
}




